﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace FlowerShopView
{
	public partial class FormMakeSale : Form
	{
		private readonly ILogger<FormMakeSale> _logger;

		private readonly IFlowerLogic _logicFlower;

		private readonly IShopLogic _logicShop;
		public FormMakeSale(ILogger<FormMakeSale> logger, IFlowerLogic logicFlower, IShopLogic logicShop)
		{
			InitializeComponent();
			_logger = logger;
			_logicFlower = logicFlower;
			_logicShop = logicShop;
		}
		private void FormMakeSale_Load(object sender, EventArgs e)
		{
			_logger.LogInformation("Flowers loading");
			try
			{
				var list = _logicFlower.ReadList(null);
				if (list != null)
				{
					comboBoxFlower.DisplayMember = "FlowerName";
					comboBoxFlower.ValueMember = "Id";
					comboBoxFlower.DataSource = list;
					comboBoxFlower.SelectedItem = null;
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Flowers loading error");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonSale_Click(object sender, EventArgs e)
		{
			if (comboBoxFlower.SelectedValue == null)
			{
				MessageBox.Show("Выберите цветы", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(textBoxCount.Text))
			{
				MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Flowers sale");
			try
			{
				var operationResult = _logicShop.MakeSale(
					new FlowerBindingModel
					{
						Id = Convert.ToInt32(comboBoxFlower.SelectedValue)
					},
					Convert.ToInt32(textBoxCount.Text)
				);
				if (!operationResult)
				{
					throw new Exception("Ошибка при продаже.");
				}
				MessageBox.Show("Продажа прошла успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Flowers sale error");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
