﻿using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FlowerShopView
{
    public partial class FormMakeSupply : Form
    {
        private readonly ILogger _logger;

        private readonly IFlowerLogic _logicFlower;

        private readonly IShopLogic _logicShop;

        public FormMakeSupply(ILogger<FormMakeSupply> logger, IFlowerLogic logicFlower, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicFlower = logicFlower;
            _logicShop = logicShop;
        }

        private void FormMakeShipment_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Flowers loading");
            try
            {
                var list = _logicFlower.ReadList(null);
                if (list != null)
                {
                    comboBoxFlower.DisplayMember = "FlowerName";
                    comboBoxFlower.ValueMember = "Id";
                    comboBoxFlower.DataSource = list;
                    comboBoxFlower.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Flowers loading error");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            _logger.LogInformation("Shops loading");
            try
            {
                var list = _logicShop.ReadList(null);
                if (list != null)
                {
                    comboBoxShop.DisplayMember = "ShopName";
                    comboBoxShop.ValueMember = "Id";
                    comboBoxShop.DataSource = list;
                    comboBoxShop.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Shops loading error");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxFlower.SelectedValue == null)
            {
                MessageBox.Show("Выберите цветок", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Shop replenishment");
            try
            {
                var iceCream = _logicFlower.ReadElement(new FlowerSearchModel
                { Id = Convert.ToInt32(comboBoxFlower.SelectedValue) });
                if (iceCream == null)
                {
                    throw new Exception("Цветок не найден.");
                }
                var operationResult = _logicShop.MakeSupply(new ShopSearchModel
                {
                    Id = Convert.ToInt32(comboBoxShop.SelectedValue)
                }, 
                iceCream, Convert.ToInt32(textBoxCount.Text));
                if (!operationResult)
                {
                    throw new Exception("Ошибка при проведении поставки.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Shop supply error");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            DialogResult = DialogResult.OK;
            Close();
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
