﻿
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;
using FlowerShopFileImplement.Models;

namespace FlowerShopFileImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		private readonly DataFileSingleton source;

		public ShopStorage()
		{
			source = DataFileSingleton.GetInstance();
		}

		public List<ShopViewModel> GetFullList()
		{
			return source.Shops
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}
			return source.Shops
					.Where(x => x.ShopName.Contains(model.ShopName))
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
			{
				return null;
			}
			return source.Shops
					.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
										(model.Id.HasValue && x.Id == model.Id))
					?.GetViewModel;
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
			var newShop = Shop.Create(model);
			if (newShop == null)
			{
				return null;
			}
			source.Shops.Add(newShop);
			source.SaveShops();
			return newShop.GetViewModel;
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (shop == null)
			{
				return null;
			}
			shop.Update(model);
			source.SaveShops();
			return shop.GetViewModel;
		}

		public ShopViewModel? Delete(ShopBindingModel model)
		{
			var element = source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				source.Shops.Remove(element);
				source.SaveShops();
				return element.GetViewModel;
			}
			return null;
		}

		public bool MakeSale(IFlowerModel model, int count)
		{
			var flower = source.Flowers.FirstOrDefault(x => x.Id == model.Id);
			int countInShops = source.Shops.SelectMany(x => x.ShopFlowers).Sum(y => y.Key == model.Id ? y.Value.Item2 : 0);

			if (flower == null || countInShops < count)
			{
				return false;
			}
			foreach (var shop in source.Shops)
			{
				var shopFlowers = shop.ShopFlowers.Where(x => x.Key == model.Id);
				if (shopFlowers.Any())
				{
					var shopFlower = shopFlowers.First();
					int min = Math.Min(shopFlower.Value.Item2, count);
					if (min == shopFlower.Value.Item2)
					{
						shop.ShopFlowers.Remove(shopFlower.Key);
					}
					else
					{
						shop.ShopFlowers[shopFlower.Key] = (shopFlower.Value.Item1, shopFlower.Value.Item2 - min);
					}
					shop.Update(new ShopBindingModel
					{
						Id = shop.Id,
						ShopName = shop.ShopName,
						Address = shop.Address,
						DateOpening = shop.DateOpening,
						ShopFlowers = shop.ShopFlowers,
						MaximumFlowers = shop.MaximumFlowers
					});
					count -= min;
					if (count <= 0)
					{
						break;
					}
				}
			}
			source.SaveShops();
			return true;
		}
	}
}
