﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;

namespace FlowerShopListImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; } = string.Empty;
        public string Address { get; private set; } = string.Empty;
        public DateTime DateOpening { get; private set; }
        public Dictionary<int, (IFlowerModel, int)> ShopFlowers { get; private set; } = new();
		public int MaximumFlowers { get; private set; }

		public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpening = model.DateOpening,
                ShopFlowers = model.ShopFlowers,
                MaximumFlowers = model.MaximumFlowers,
            };
        }

        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpening = model.DateOpening;
            ShopFlowers = model.ShopFlowers;
            MaximumFlowers = model.MaximumFlowers;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpening = DateOpening,
            ShopFlowers = ShopFlowers,
            MaximumFlowers = MaximumFlowers,
        };
    }
}
